@@ -33,7 +33,7 @@
 #endif
 
 #define INBOX_DIR		"inbox"
-#define OUTBOX_DIR		"outbox"
+#define OUTBOX_DIR		"sent"
 #define QUEUE_DIR		"queue"
 #define DRAFT_DIR		"draft"
 #define TRASH_DIR		"trash"
