@@ -131,7 +131,7 @@ struct _Folder
 	gboolean (*is_msg_changed)	(Folder		*folder,
 					 FolderItem	*item,
 					 MsgInfo	*msginfo);
-	void     (*scan)		(Folder		*folder,
+	gint     (*scan)		(Folder		*folder,
 					 FolderItem	*item);
 	void     (*scan_tree)		(Folder		*folder);
 
@@ -194,10 +194,14 @@ struct _FolderItem
 	guint collapsed : 1; /* collapsed item       */
 	guint threaded  : 1; /* threaded folder view */
 
+	guint opened    : 1; /* opened by summary view */
+
 	FolderItem *parent;
 
 	Folder *folder;
 
+	GSList *mark_queue;
+
 	gpointer data;
 };
 
@@ -239,8 +243,14 @@ void   folder_count_total_msgs	(guint		*new,
 				 guint		*unread,
 				 guint		*total);
 
-Folder     *folder_find_from_path	(const gchar	*path);
-FolderItem *folder_find_item_from_path	(const gchar	*path);
+Folder     *folder_find_from_path		(const gchar	*path);
+Folder     *folder_find_from_name		(const gchar	*name,
+						 FolderType	 type);
+FolderItem *folder_find_item_from_path		(const gchar	*path);
+gchar      *folder_get_identifier		(Folder		*folder);
+gchar      *folder_item_get_identifier		(FolderItem	*item);
+FolderItem *folder_find_item_from_identifier	(const gchar	*identifier);
+
 Folder     *folder_get_default_folder	(void);
 FolderItem *folder_get_default_inbox	(void);
 FolderItem *folder_get_default_outbox	(void);
@@ -248,8 +258,10 @@ FolderItem *folder_get_default_draft	(void);
 FolderItem *folder_get_default_queue	(void);
 FolderItem *folder_get_default_trash	(void);
 
+void folder_set_missing_folders		(void);
+
 gchar *folder_item_get_path		(FolderItem	*item);
-void   folder_item_scan			(FolderItem	*item);
+gint   folder_item_scan			(FolderItem	*item);
 void   folder_item_scan_foreach		(GHashTable	*table);
 gchar *folder_item_fetch_msg		(FolderItem	*item,
 					 gint		 num);
