@@ -55,6 +55,7 @@
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "prefs_filter.h"
+#include "prefs_folder_item.h"
 #include "account.h"
 #include "folder.h"
 #include "inc.h"
@@ -168,9 +169,6 @@ static void folderview_remove_mailbox_cb(FolderView	*folderview,
 static void folderview_new_imap_folder_cb(FolderView	*folderview,
 					  guint		 action,
 					  GtkWidget	*widget);
-static void folderview_rm_imap_folder_cb (FolderView	*folderview,
-					  guint		 action,
-					  GtkWidget	*widget);
 static void folderview_rm_imap_server_cb (FolderView	*folderview,
 					  guint		 action,
 					  GtkWidget	*widget);
@@ -189,6 +187,10 @@ static void folderview_search_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
 
+static void folderview_property_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  GdkDragContext *context,
 					  gint            x,
@@ -221,14 +223,14 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
-	{N_("/_Property..."),		NULL, NULL, 0, NULL}
+	{N_("/_Property..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
 static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 {
 	{N_("/Create _new folder..."),	NULL, folderview_new_imap_folder_cb, 0, NULL},
-	{N_("/_Rename folder..."),	NULL, NULL, 0, NULL},
-	{N_("/_Delete folder"),		NULL, folderview_rm_imap_folder_cb, 0, NULL},
+	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb,   0, NULL},
+	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb,   0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
@@ -237,7 +239,7 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/Remove _IMAP4 account"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
-	{N_("/_Property..."),		NULL, NULL, 0, NULL}
+	{N_("/_Property..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
 static GtkItemFactoryEntry folderview_news_popup_entries[] =
@@ -252,7 +254,7 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 	{N_("/Remove _news account"),	NULL, folderview_rm_news_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
-	{N_("/_Property..."),		NULL, NULL, 0, NULL}
+	{N_("/_Property..."),		NULL, folderview_property_cb, 0, NULL}
 };
 
 
@@ -702,7 +704,10 @@ void folderview_check_new(Folder *folder)
 			if (!folder && !FOLDER_IS_LOCAL(item->folder)) continue;
 
 			folderview_scan_tree_func(item->folder, item, NULL);
-			folder_item_scan(item);
+			if (folder_item_scan(item) < 0) {
+				if (folder && !FOLDER_IS_LOCAL(folder))
+					break;
+			}
 			folderview_update_node(folderview, node);
 		}
 
@@ -825,29 +830,39 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 	case F_INBOX:
 		xpm = openxpm = inboxxpm;
 		mask = openmask = inboxxpmmask;
-		name = g_strdup(_("Inbox"));
+		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
+				!strcmp2(item->name, INBOX_DIR) ? _("Inbox") :
+				item->name);
 		break;
 	case F_OUTBOX:
 		xpm = openxpm = outboxxpm;
 		mask = openmask = outboxxpmmask;
-		name = g_strdup(_("Outbox"));
+		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
+				!strcmp2(item->name, OUTBOX_DIR) ? _("Sent") :
+				item->name);
 		break;
 	case F_QUEUE:
 		xpm = openxpm = outboxxpm;
 		mask = openmask = outboxxpmmask;
-		name = g_strdup(_("Queue"));
+		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
+				!strcmp2(item->name, QUEUE_DIR) ? _("Queue") :
+				item->name);
 		break;
 	case F_TRASH:
 		xpm = openxpm = trashxpm;
 		mask = openmask = trashxpmmask;
-		name = g_strdup(_("Trash"));
+		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
+				!strcmp2(item->name, TRASH_DIR) ? _("Trash") :
+				item->name);
 		break;
 	case F_DRAFT:
 		xpm = folderxpm;
 		mask = folderxpmmask;
 		openxpm = folderopenxpm;
 		openmask = folderopenxpmmask;
-		name = g_strdup(_("Draft"));
+		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
+				!strcmp2(item->name, DRAFT_DIR) ? _("Drafts") :
+				item->name);
 		break;
 	default:
 		xpm = folderxpm;
@@ -866,8 +881,13 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 				name = "";
 			}
 			name = g_strconcat(item->name, name, NULL);
-		} else
-			name = g_strdup(item->name);
+		} else {
+			if (item->folder->type == F_NEWS &&
+			    !strcmp2(item->name, item->path))
+				name = get_abbrev_newsgroup_name(item->path);
+			else
+				name = g_strdup(item->name);
+		}
 	}
 
 	if (!GTK_CTREE_ROW(node)->expanded &&
@@ -912,7 +932,8 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 		gtk_ctree_node_set_text(ctree, node, COL_TOTAL,  itos(item->total));
 	}
 
-	if (item->stype == F_TRASH) {
+	if (item->stype == F_OUTBOX || item->stype == F_DRAFT ||
+	    item->stype == F_TRASH) {
 		use_bold = use_color = FALSE;
 	} else if (item->stype == F_QUEUE) {
 		/* highlight queue folder if there are any messages */
@@ -1012,18 +1033,19 @@ static void folderview_expand_func(GtkCTree *ctree, GtkCTreeNode *node,
 #define SET_SPECIAL_FOLDER(ctree, item) \
 { \
 	if (item) { \
-		GtkCTreeNode *node, *sibling; \
+		GtkCTreeNode *node, *parent, *sibling; \
  \
 		node = gtk_ctree_find_by_row_data(ctree, root, item); \
 		if (!node) \
 			g_warning("%s not found.\n", item->path); \
 		else { \
-			if (!prev) \
-				sibling = GTK_CTREE_ROW(root)->children; \
-			else \
+			parent = GTK_CTREE_ROW(node)->parent; \
+			if (prev && parent == GTK_CTREE_ROW(prev)->parent) \
 				sibling = GTK_CTREE_ROW(prev)->sibling; \
+			else \
+				sibling = GTK_CTREE_ROW(parent)->children; \
 			if (node != sibling) \
-				gtk_ctree_move(ctree, node, root, sibling); \
+				gtk_ctree_move(ctree, node, parent, sibling); \
 		} \
  \
 		prev = node; \
@@ -1104,9 +1126,9 @@ void folderview_rename_folder(FolderView *folderview)
 	case F_MH:
 	case F_MBOX:
 	case F_MAILDIR:
+	case F_IMAP:
 		folderview_rename_folder_cb(folderview, 0, NULL);
 		break;
-	case F_IMAP:
 	case F_NEWS:
 	default:
 		break;
@@ -1130,10 +1152,9 @@ void folderview_delete_folder(FolderView *folderview)
 	case F_MH:
 	case F_MBOX:
 	case F_MAILDIR:
+	case F_IMAP:
 		folderview_delete_folder_cb(folderview, 0, NULL);
 		break;
-	case F_IMAP:
-		folderview_rm_imap_folder_cb(folderview, 0, NULL);
 	case F_NEWS:
 	default:
 		break;
@@ -1151,13 +1172,14 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	FolderItem *item;
 	Folder *folder;
 	GtkWidget *popup;
-	gboolean new_folder    = FALSE;
-	gboolean rename_folder = FALSE;
-	gboolean delete_folder = FALSE;
-	gboolean update_tree   = FALSE;
-	gboolean rescan_tree   = FALSE;
-	gboolean remove_tree   = FALSE;
-	gboolean search_folder = FALSE;
+	gboolean new_folder      = FALSE;
+	gboolean rename_folder   = FALSE;
+	gboolean delete_folder   = FALSE;
+	gboolean update_tree     = FALSE;
+	gboolean rescan_tree     = FALSE;
+	gboolean remove_tree     = FALSE;
+	gboolean search_folder   = FALSE;
+	gboolean folder_property = FALSE;
 
 	if (!event) return;
 
@@ -1204,7 +1226,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		if (item->parent == NULL)
 			update_tree = remove_tree = TRUE;
 		else
-			search_folder = TRUE;
+			search_folder = folder_property = TRUE;
 		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
 			if (item->parent == NULL)
 				update_tree = rescan_tree = TRUE;
@@ -1229,6 +1251,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(mail_factory, "/Rescan folder tree", rescan_tree);
 		SET_SENS(mail_factory, "/Remove mailbox", remove_tree);
 		SET_SENS(mail_factory, "/Search messages...", search_folder);
+		SET_SENS(mail_factory, "/Property...", folder_property);
 	} else if (FOLDER_TYPE(folder) == F_IMAP) {
 		popup = folderview->imap_popup;
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
@@ -1239,6 +1262,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(imap_factory, "/Rescan folder tree", rescan_tree);
 		SET_SENS(imap_factory, "/Remove IMAP4 account", remove_tree);
 		SET_SENS(imap_factory, "/Search messages...", search_folder);
+		SET_SENS(imap_factory, "/Property...", folder_property);
 	} else if (FOLDER_TYPE(folder) == F_NEWS) {
 		popup = folderview->news_popup;
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
@@ -1247,6 +1271,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(news_factory, "/Check for new messages", update_tree);
 		SET_SENS(news_factory, "/Remove news account", remove_tree);
 		SET_SENS(news_factory, "/Search messages...", search_folder);
+		SET_SENS(news_factory, "/Property...", folder_property);
 	} else
 		return;
 
@@ -1521,7 +1546,8 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	gchar *new_folder;
 	gchar *message;
 	gchar *old_path;
-	gchar *new_path;
+	gchar *old_id;
+	gchar *new_id;
 
 	if (!folderview->selected) return;
 
@@ -1554,24 +1580,21 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	}
 
 	Xstrdup_a(old_path, item->path, {g_free(new_folder); return;});
+	old_id = folder_item_get_identifier(item);
 
 	if (item->folder->rename_folder(item->folder, item, new_folder) < 0) {
+		g_free(old_id);
 		g_free(new_folder);
 		return;
 	}
-
-	if (strchr(item->path, G_DIR_SEPARATOR) != NULL) {
-		gchar *dirname;
-		dirname = g_dirname(item->path);
-		new_path = g_strconcat(dirname, G_DIR_SEPARATOR_S, new_folder, NULL);
-		g_free(dirname);
-	} else
-		new_path = g_strdup(new_folder);
-
 	g_free(new_folder);
 
-	prefs_filter_rename_path(old_path, new_path);
-	g_free(new_path);
+	if (folder_get_default_folder() == item->folder)
+		prefs_filter_rename_path(old_path, item->path);
+	new_id = folder_item_get_identifier(item);
+	prefs_filter_rename_path(old_id, new_id);
+	g_free(old_id);
+	g_free(new_id);
 
 	gtk_clist_freeze(GTK_CLIST(ctree));
 
@@ -1579,10 +1602,13 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	folderview_sort_folders(folderview,
 				GTK_CTREE_ROW(folderview->selected)->parent,
 				item->folder);
-	if (folderview->opened == folderview->selected) {
-		if (!GTK_CTREE_ROW(folderview->opened)->children)
-			gtk_ctree_expand(ctree, folderview->opened);
-		summary_show(folderview->summaryview, item, FALSE);
+	if (folderview->opened == folderview->selected ||
+	    gtk_ctree_is_ancestor(ctree,
+				  folderview->selected,
+				  folderview->opened)) {
+		GtkCTreeNode *node = folderview->opened;
+		folderview_unselect(folderview);
+		folderview_select_node(folderview, node);
 	}
 
 	gtk_clist_thaw(GTK_CLIST(ctree));
@@ -1598,6 +1624,7 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	gchar *message;
 	AlertValue avalue;
 	gchar *old_path;
+	gchar *old_id;
 
 	if (!folderview->selected) return;
 
@@ -1616,14 +1643,23 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	if (avalue != G_ALERTDEFAULT) return;
 
 	Xstrdup_a(old_path, item->path, return);
+	old_id = folder_item_get_identifier(item);
 
 	if (item->folder->remove_folder(item->folder, item) < 0) {
 		alertpanel_error(_("Can't remove the folder `%s'."),
 				 item->path);
+		if (folderview->opened == folderview->selected)
+			summary_show(folderview->summaryview,
+				     folderview->summaryview->folder_item,
+				     FALSE);
+		g_free(old_id);
 		return;
 	}
 
-	prefs_filter_delete_path(old_path);
+	if (folder_get_default_folder() == item->folder)
+		prefs_filter_delete_path(old_path);
+	prefs_filter_delete_path(old_id);
+	g_free(old_id);
 
 	if (folderview->opened == folderview->selected ||
 	    gtk_ctree_is_ancestor(ctree,
@@ -1738,51 +1774,6 @@ static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
 	folder_write_list();
 }
 
-static void folderview_rm_imap_folder_cb(FolderView *folderview, guint action,
-					 GtkWidget *widget)
-{
-	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	FolderItem *item;
-	gchar *message;
-	AlertValue avalue;
-
-	if (!folderview->selected) return;
-
-	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_IMAP);
-	g_return_if_fail(item->folder->account != NULL);
-
-	message = g_strdup_printf(_("Really delete folder `%s'?"),
-				  g_basename(item->path));
-	avalue = alertpanel(_("Delete folder"), message,
-			    _("Yes"), _("+No"), NULL);
-	g_free(message);
-	if (avalue != G_ALERTDEFAULT) return;
-
-	if (item->folder->remove_folder(item->folder, item) < 0) {
-		alertpanel_error(_("Can't remove the folder `%s'."),
-				 item->path);
-		if (folderview->opened == folderview->selected)
-			summary_show(folderview->summaryview,
-				     folderview->summaryview->folder_item,
-				     FALSE);
-		return;
-	}
-
-	if (folderview->opened == folderview->selected ||
-	    gtk_ctree_is_ancestor(ctree,
-				  folderview->selected,
-				  folderview->opened)) {
-		summary_clear_all(folderview->summaryview);
-		folderview->opened = NULL;
-	}
-
-	gtk_ctree_remove_node(ctree, folderview->selected);
-	folder_write_list();
-}
-
 static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 					 GtkWidget *widget)
 {
@@ -1993,6 +1984,21 @@ static void folderview_search_cb(FolderView *folderview, guint action,
 	summary_search(folderview->summaryview);
 }
 
+static void folderview_property_cb(FolderView *folderview, guint action,
+				   GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+
+	prefs_folder_item_open(item);
+}
+
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  GdkDragContext *context,
 					  gint            x,
