@@ -31,6 +31,9 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <time.h>
+#if HAVE_LIBJCONV
+#  include <jconv.h>
+#endif
 
 #include "intl.h"
 #include "imap.h"
@@ -80,8 +83,13 @@ static gint imap_scan_tree_recursive	(IMAPSession	*session,
 					 FolderItem	*item,
 					 IMAPNameSpace	*namespace);
 static GSList *imap_parse_list		(IMAPSession	*session,
-					 const gchar	*path);
-static gint imap_create_trash		(Folder		*folder);
+					 const gchar	*real_path);
+
+static void imap_create_missing_folders	(Folder			*folder);
+static FolderItem *imap_create_special_folder
+					(Folder			*folder,
+					 SpecialFolderItemType	 stype,
+					 const gchar		*name);
 
 static gint imap_do_copy		(Folder		*folder,
 					 FolderItem	*dest,
@@ -133,8 +141,9 @@ static gint imap_status			(IMAPSession	*session,
 					 const gchar	*path,
 					 gint		*messages,
 					 gint		*recent,
-					 gint		*unseen,
-					 guint32	*uid_validity);
+					 guint32	*uid_next,
+					 guint32	*uid_validity,
+					 gint		*unseen);
 
 static void imap_parse_namespace		(IMAPSession	*session,
 						 IMAPFolder	*folder);
@@ -195,6 +204,9 @@ static gint imap_cmd_examine	(SockInfo	*sock,
 				 guint32	*uid_validity);
 static gint imap_cmd_create	(SockInfo	*sock,
 				 const gchar	*folder);
+static gint imap_cmd_rename	(SockInfo	*sock,
+				 const gchar	*oldfolder,
+				 const gchar	*newfolder);
 static gint imap_cmd_delete	(SockInfo	*sock,
 				 const gchar	*folder);
 static gint imap_cmd_envelope	(SockInfo	*sock,
@@ -243,6 +255,12 @@ static gchar *search_array_str			(GPtrArray	*array,
 static void imap_path_separator_subst		(gchar		*str,
 						 gchar		 separator);
 
+static gchar *imap_modified_utf7_to_locale	(const gchar	*mutf7_str);
+static gchar *imap_locale_to_modified_utf7	(const gchar	*from);
+
+static gboolean imap_rename_folder_func		(GNode		*node,
+						 gpointer	 data);
+
 
 Folder *imap_folder_new(const gchar *name, const gchar *path)
 {
@@ -275,10 +293,12 @@ static void imap_folder_init(Folder *folder, const gchar *name,
 	folder->copy_msgs_with_dest = imap_copy_msgs_with_dest;
 	folder->remove_msg          = imap_remove_msg;
 	folder->remove_all_msg      = imap_remove_all_msg;
+	folder->is_msg_changed      = imap_is_msg_changed;
 	folder->scan                = imap_scan_folder;
 	folder->scan_tree           = imap_scan_tree;
 	folder->create_tree         = imap_create_tree;
 	folder->create_folder       = imap_create_folder;
+	folder->rename_folder       = imap_rename_folder;
 	folder->remove_folder       = imap_remove_folder;
 }
 
@@ -424,7 +444,7 @@ void imap_session_destroy_all(void)
 		IMAPSession *session = (IMAPSession *)session_list->data;
 
 		imap_cmd_logout(SESSION(session)->sock);
-		imap_session_destroy(session);
+		session_destroy(SESSION(session));
 	}
 }
 
@@ -550,6 +570,15 @@ gchar *imap_fetch_msg(Folder *folder, FolderItem *item, gint uid)
 		return NULL;
 	}
 
+	ok = imap_select(session, IMAP_FOLDER(folder), item->path,
+			 NULL, NULL, NULL, NULL);
+	statusbar_pop_all();
+	if (ok != IMAP_SUCCESS) {
+		g_warning(_("can't select mailbox %s\n"), item->path);
+		g_free(filename);
+		return NULL;
+	}
+
 	debug_print(_("getting message %d...\n"), uid);
 	ok = imap_cmd_fetch(SESSION(session)->sock, (guint32)uid, filename);
 
@@ -877,34 +906,43 @@ gint imap_remove_all_msg(Folder *folder, FolderItem *item)
 	return IMAP_SUCCESS;
 }
 
-void imap_scan_folder(Folder *folder, FolderItem *item)
+gboolean imap_is_msg_changed(Folder *folder, FolderItem *item, MsgInfo *msginfo)
+{
+	/* TODO: properly implement this method */
+	return FALSE;
+}
+
+gint imap_scan_folder(Folder *folder, FolderItem *item)
 {
 	IMAPSession *session;
 	gint messages, recent, unseen;
-	guint32 uid_validity;
+	guint32 uid_next, uid_validity;
 	gint ok;
 
-	g_return_if_fail(folder != NULL);
-	g_return_if_fail(item != NULL);
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(item != NULL, -1);
 
 	session = imap_session_get(folder);
-	if (!session) return;
+	if (!session) return -1;
 
 	ok = imap_status(session, IMAP_FOLDER(folder), item->path,
-			 &messages, &recent, &unseen, &uid_validity);
+			 &messages, &recent, &uid_next, &uid_validity, &unseen);
 	statusbar_pop_all();
-	if (ok != IMAP_SUCCESS) return;
+	if (ok != IMAP_SUCCESS) return -1;
 
-	item->new = recent;
+	item->new = unseen > 0 ? recent : 0;
 	item->unread = unseen;
 	item->total = messages;
+	item->last_num = (messages > 0 && uid_next > 0) ? uid_next - 1 : 0;
 	/* item->mtime = uid_validity; */
+
+	return 0;
 }
 
 void imap_scan_tree(Folder *folder)
 {
 	IMAPFolder *imapfolder = IMAP_FOLDER(folder);
-	FolderItem *item, *inbox;
+	FolderItem *item;
 	IMAPSession *session;
 	IMAPNameSpace *namespace = NULL;
 	gchar *root_folder = NULL;
@@ -940,19 +978,13 @@ void imap_scan_tree(Folder *folder)
 
 	imap_scan_tree_recursive(session, item, namespace);
 
-	if (!folder->inbox) {
-		inbox = folder_item_new("INBOX", "INBOX");
-		inbox->stype = F_INBOX;
-		folder_item_append(item, inbox);
-		folder->inbox = inbox;
-	}
-	if (!folder->trash)
-		imap_create_trash(folder);
+	imap_create_missing_folders(folder);
 }
 
 static gint imap_scan_tree_recursive(IMAPSession *session, FolderItem *item,
 				     IMAPNameSpace *namespace)
 {
+	Folder *folder;
 	IMAPFolder *imapfolder;
 	FolderItem *new_item;
 	GSList *item_list, *cur;
@@ -965,14 +997,14 @@ static gint imap_scan_tree_recursive(IMAPSession *session, FolderItem *item,
 	g_return_val_if_fail(item->folder != NULL, -1);
 	g_return_val_if_fail(item->no_sub == FALSE, -1);
 
-	imapfolder = IMAP_FOLDER(item->folder);
+	folder = FOLDER(item->folder);
+	imapfolder = IMAP_FOLDER(folder);
 
 	if (namespace && namespace->separator)
 		separator = namespace->separator;
 
 	if (item->folder->ui_func)
-		item->folder->ui_func(item->folder, item,
-				      item->folder->ui_func_data);
+		item->folder->ui_func(folder, item, folder->ui_func_data);
 
 	if (item->path) {
 		wildcard[0] = separator;
@@ -982,7 +1014,8 @@ static gint imap_scan_tree_recursive(IMAPSession *session, FolderItem *item,
 	} else {
 		wildcard[0] = '%';
 		wildcard[1] = '\0';
-		real_path = g_strdup(namespace && namespace->name
+		real_path = g_strdup(namespace && namespace->name &&
+				     strncmp(namespace->name, "INBOX", 5) != 0
 				     ? namespace->name : "");
 	}
 
@@ -1000,22 +1033,35 @@ static gint imap_scan_tree_recursive(IMAPSession *session, FolderItem *item,
 	for (cur = item_list; cur != NULL; cur = cur->next) {
 		new_item = cur->data;
 		if (!strcmp(new_item->path, "INBOX")) {
-			if (!item->folder->inbox) {
+			if (!folder->inbox) {
 				new_item->stype = F_INBOX;
 				item->folder->inbox = new_item;
 			} else {
 				folder_item_destroy(new_item);
 				continue;
 			}
-		} else if (!item->parent && !item->folder->trash) {
-			if (!strcasecmp(g_basename(new_item->path), "Trash")) {
+		} else if (!item->parent || item->stype == F_INBOX) {
+			gchar *base;
+
+			base = g_basename(new_item->path);
+
+			if (!folder->outbox && !strcasecmp(base, "Sent")) {
+				new_item->stype = F_OUTBOX;
+				folder->outbox = new_item;
+			} else if (!folder->draft && !strcasecmp(base, "Drafts")) {
+				new_item->stype = F_DRAFT;
+				folder->draft = new_item;
+			} else if (!folder->queue && !strcasecmp(base, "Queue")) {
+				new_item->stype = F_QUEUE;
+				folder->queue = new_item;
+			} else if (!folder->trash && !strcasecmp(base, "Trash")) {
 				new_item->stype = F_TRASH;
-				item->folder->trash = new_item;
+				folder->trash = new_item;
 			}
 		}
 		folder_item_append(item, new_item);
 		if (new_item->no_select == FALSE)
-			imap_scan_folder(new_item->folder, new_item);
+			imap_scan_folder(folder, new_item);
 		if (new_item->no_sub == FALSE)
 			imap_scan_tree_recursive(session, new_item, namespace);
 	}
@@ -1023,18 +1069,20 @@ static gint imap_scan_tree_recursive(IMAPSession *session, FolderItem *item,
 	return IMAP_SUCCESS;
 }
 
-static GSList *imap_parse_list(IMAPSession *session, const gchar *path)
+static GSList *imap_parse_list(IMAPSession *session, const gchar *real_path)
 {
 	gchar buf[IMAPBUFSIZE];
 	gchar flags[256];
 	gchar separator[16];
 	gchar *p;
 	gchar *name;
+	gchar *loc_name, *loc_path;
 	GSList *item_list = NULL;
 	GString *str;
 	FolderItem *new_item;
 
-	debug_print("getting list of %s ...\n", *path ? path : "\"\"");
+	debug_print("getting list of %s ...\n",
+		    *real_path ? real_path : "\"\"");
 
 	str = g_string_new(NULL);
 
@@ -1076,22 +1124,27 @@ static GSList *imap_parse_list(IMAPSession *session, const gchar *path)
 			strncpy2(buf, p, sizeof(buf));
 		strtailchomp(buf, separator[0]);
 		if (buf[0] == '\0') continue;
-		if (!strcmp(buf, path)) continue;
+		if (!strcmp(buf, real_path)) continue;
 
 		if (separator[0] != '\0')
 			subst_char(buf, separator[0], '/');
 		name = g_basename(buf);
 		if (name[0] == '.') continue;
 
-		new_item = folder_item_new(name, buf);
+		loc_name = imap_modified_utf7_to_locale(name);
+		loc_path = imap_modified_utf7_to_locale(buf);
+		new_item = folder_item_new(loc_name, loc_path);
 		if (strcasestr(flags, "\\Noinferiors") != NULL)
 			new_item->no_sub = TRUE;
-		if (strcasestr(flags, "\\Noselect") != NULL)
+		if (strcmp(buf, "INBOX") != 0 &&
+		    strcasestr(flags, "\\Noselect") != NULL)
 			new_item->no_select = TRUE;
 
 		item_list = g_slist_append(item_list, new_item);
 
-		debug_print("folder %s has been added.\n", buf);
+		debug_print("folder %s has been added.\n", loc_path);
+		g_free(loc_path);
+		g_free(loc_name);
 	}
 
 	g_string_free(str, TRUE);
@@ -1102,97 +1155,75 @@ static GSList *imap_parse_list(IMAPSession *session, const gchar *path)
 
 gint imap_create_tree(Folder *folder)
 {
-	FolderItem *item;
-
 	g_return_val_if_fail(folder != NULL, -1);
 	g_return_val_if_fail(folder->node != NULL, -1);
 	g_return_val_if_fail(folder->node->data != NULL, -1);
 	g_return_val_if_fail(folder->account != NULL, -1);
 
 	imap_scan_tree(folder);
-
-	item = FOLDER_ITEM(folder->node->data);
-
-	if (!folder->inbox) {
-		FolderItem *inbox;
-
-		inbox = folder_item_new("INBOX", "INBOX");
-		inbox->stype = F_INBOX;
-		folder_item_append(item, inbox);
-		folder->inbox = inbox;
-	}
-	if (!folder->trash)
-		imap_create_trash(folder);
+	imap_create_missing_folders(folder);
 
 	return 0;
 }
 
-static gint imap_create_trash(Folder *folder)
+static void imap_create_missing_folders(Folder *folder)
+{
+	g_return_if_fail(folder != NULL);
+
+	if (!folder->inbox)
+		folder->inbox = imap_create_special_folder
+			(folder, F_INBOX, "INBOX");
+#if 0
+	if (!folder->outbox)
+		folder->outbox = imap_create_special_folder
+			(folder, F_OUTBOX, "Sent");
+	if (!folder->draft)
+		folder->draft = imap_create_special_folder
+			(folder, F_DRAFT, "Drafts");
+	if (!folder->queue)
+		folder->queue = imap_create_special_folder
+			(folder, F_QUEUE, "Queue");
+#endif
+	if (!folder->trash)
+		folder->trash = imap_create_special_folder
+			(folder, F_TRASH, "Trash");
+}
+
+static FolderItem *imap_create_special_folder(Folder *folder,
+					      SpecialFolderItemType stype,
+					      const gchar *name)
 {
-	IMAPFolder *imapfolder = IMAP_FOLDER(folder);
 	FolderItem *item;
 	FolderItem *new_item;
-	gchar *trash_path;
-	gchar *imap_dir = "";
 
-	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(folder->node != NULL, -1);
-	g_return_val_if_fail(folder->node->data != NULL, -1);
-	g_return_val_if_fail(folder->account != NULL, -1);
-
-	if (folder->account->imap_dir && *folder->account->imap_dir) {
-		gchar *tmpdir;
-
-		Xstrdup_a(tmpdir, folder->account->imap_dir, return -1);
-		strtailchomp(tmpdir, '/');
-		Xalloca(imap_dir, strlen(tmpdir) + 2, return -1);
-		g_snprintf(imap_dir, strlen(tmpdir) + 2, "%s%c", tmpdir, '/');
-	}
-
-	if (imapfolder->namespace && imapfolder->namespace->data) {
-		IMAPNameSpace *namespace =
-			(IMAPNameSpace *)imapfolder->namespace->data;
-
-		if (*namespace->name != '\0') {
-			gchar *name;
-
-			Xstrdup_a(name, namespace->name, return -1);
-			subst_char(name, namespace->separator, '/');
-			trash_path = g_strconcat(name, imap_dir, "Trash", NULL);
-		} else
-			trash_path = g_strconcat(imap_dir, "Trash", NULL);
-	} else
-		trash_path = g_strconcat(imap_dir, "Trash", NULL);
+	g_return_val_if_fail(folder != NULL, NULL);
+	g_return_val_if_fail(folder->node != NULL, NULL);
+	g_return_val_if_fail(folder->node->data != NULL, NULL);
+	g_return_val_if_fail(folder->account != NULL, NULL);
+	g_return_val_if_fail(name != NULL, NULL);
 
 	item = FOLDER_ITEM(folder->node->data);
-	new_item = imap_create_folder(folder, item, trash_path);
+	new_item = imap_create_folder(folder, item, name);
 
 	if (!new_item) {
-		gchar *path;
+		g_warning(_("Can't create '%s'\n"), name);
+		if (!folder->inbox) return NULL;
 
-		new_item = folder_item_new("Trash", trash_path);
-		folder_item_append(item, new_item);
+		new_item = imap_create_folder(folder, folder->inbox, name);
+		if (!new_item)
+			g_warning(_("Can't create '%s' under INBOX\n"), name);
+		else
+			new_item->stype = stype;
+	} else
+		new_item->stype = stype;
 
-		path = folder_item_get_path(new_item);
-		if (!is_dir_exist(path))
-			make_dir_hier(path);
-		g_free(path);
-	} else {
-		g_free(new_item->name);
-		new_item->name = g_strdup("Trash");
-	}
-	new_item->stype = F_TRASH;
-	folder->trash = new_item;
-
-	g_free(trash_path);
-
-	return 0;
+	return new_item;
 }
 
 FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 			       const gchar *name)
 {
-	gchar *dirpath, *imappath;
+	gchar *dirpath, *imap_path;
 	IMAPSession *session;
 	IMAPNameSpace *namespace;
 	FolderItem *new_item;
@@ -1221,15 +1252,15 @@ FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 	} else
 		dirpath = g_strdup(name);
 
-	Xstrdup_a(imappath, dirpath, {g_free(dirpath); return NULL;});
-	Xstrdup_a(new_name, name, {g_free(dirpath); return NULL;});
-	namespace = imap_find_namespace(IMAP_FOLDER(folder), imappath);
-	if (namespace && namespace->separator) {
-		imap_path_separator_subst(imappath, namespace->separator);
-		imap_path_separator_subst(new_name, namespace->separator);
-		strtailchomp(new_name, namespace->separator);
-	}
 	strtailchomp(dirpath, '/');
+	imap_path = imap_locale_to_modified_utf7(dirpath);
+	Xstrdup_a(new_name, name, {g_free(dirpath); return NULL;});
+	strtailchomp(new_name, '/');
+	namespace = imap_find_namespace(IMAP_FOLDER(folder), imap_path);
+	if (namespace && namespace->separator) {
+		imap_path_separator_subst(imap_path, namespace->separator);
+		subst_char(new_name, '/', namespace->separator);
+	}
 
 	if (strcmp(name, "INBOX") != 0) {
 		GPtrArray *argbuf;
@@ -1237,11 +1268,12 @@ FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 		gboolean exist = FALSE;
 
 		argbuf = g_ptr_array_new();
-		ok = imap_cmd_list(SESSION(session)->sock, NULL, imappath,
+		ok = imap_cmd_list(SESSION(session)->sock, NULL, imap_path,
 				   argbuf);
 		statusbar_pop_all();
 		if (ok != IMAP_SUCCESS) {
 			log_warning(_("can't create mailbox: LIST failed\n"));
+			g_free(imap_path);
 			g_free(dirpath);
 			g_ptr_array_free(argbuf, TRUE);
 			return NULL;
@@ -1258,10 +1290,11 @@ FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 		g_ptr_array_free(argbuf, TRUE);
 
 		if (!exist) {
-			ok = imap_cmd_create(SESSION(session)->sock, imappath);
+			ok = imap_cmd_create(SESSION(session)->sock, imap_path);
 			statusbar_pop_all();
 			if (ok != IMAP_SUCCESS) {
 				log_warning(_("can't create mailbox\n"));
+				g_free(imap_path);
 				g_free(dirpath);
 				return NULL;
 			}
@@ -1270,6 +1303,7 @@ FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 
 	new_item = folder_item_new(new_name, dirpath);
 	folder_item_append(parent, new_item);
+	g_free(imap_path);
 	g_free(dirpath);
 
 	dirpath = folder_item_get_path(new_item);
@@ -1280,11 +1314,100 @@ FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 	return new_item;
 }
 
+gint imap_rename_folder(Folder *folder, FolderItem *item, const gchar *name)
+{
+	gchar *dirpath;
+	gchar *newpath;
+	gchar *real_oldpath;
+	gchar *real_newpath;
+	GNode *node;
+	gchar *paths[2];
+	gchar *old_cache_dir;
+	gchar *new_cache_dir;
+	IMAPSession *session;
+	IMAPNameSpace *namespace;
+	gint ok;
+	gint exists, recent, unseen;
+	guint32 uid_validity;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->path != NULL, -1);
+	g_return_val_if_fail(name != NULL, -1);
+
+	session = imap_session_get(folder);
+	if (!session) return -1;
+
+	real_oldpath = imap_get_real_path(IMAP_FOLDER(folder), item->path);
+
+	g_free(session->mbox);
+	session->mbox = NULL;
+	ok = imap_cmd_examine(SESSION(session)->sock, "INBOX",
+			      &exists, &recent, &unseen, &uid_validity);
+	statusbar_pop_all();
+	if (ok != IMAP_SUCCESS) {
+		g_free(real_oldpath);
+		return -1;
+	}
+
+	namespace = imap_find_namespace(IMAP_FOLDER(folder), item->path);
+	if (strchr(item->path, G_DIR_SEPARATOR)) {
+		dirpath = g_dirname(item->path);
+		newpath = g_strconcat(dirpath, G_DIR_SEPARATOR_S, name, NULL);
+		g_free(dirpath);
+	} else
+		newpath = g_strdup(name);
+
+	real_newpath = imap_locale_to_modified_utf7(newpath);
+	if (namespace && namespace->separator)
+		imap_path_separator_subst(real_newpath, namespace->separator);
+
+	ok = imap_cmd_rename(SESSION(session)->sock, real_oldpath, real_newpath);
+	statusbar_pop_all();
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't rename mailbox: %s to %s\n"),
+			    real_oldpath, real_newpath);
+		g_free(real_oldpath);
+		g_free(newpath);
+		g_free(real_newpath);
+		return -1;
+	}
+
+	g_free(item->name);
+	item->name = g_strdup(name);
+
+	old_cache_dir = folder_item_get_path(item);
+
+	node = g_node_find(item->folder->node, G_PRE_ORDER, G_TRAVERSE_ALL,
+			   item);
+	paths[0] = g_strdup(item->path);
+	paths[1] = newpath;
+	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+			imap_rename_folder_func, paths);
+
+	if (is_dir_exist(old_cache_dir)) {
+		new_cache_dir = folder_item_get_path(item);
+		if (rename(old_cache_dir, new_cache_dir) < 0) {
+			FILE_OP_ERROR(old_cache_dir, "rename");
+		}
+		g_free(new_cache_dir);
+	}
+
+	g_free(old_cache_dir);
+	g_free(paths[0]);
+	g_free(newpath);
+	g_free(real_oldpath);
+	g_free(real_newpath);
+
+	return 0;
+}
+
 gint imap_remove_folder(Folder *folder, FolderItem *item)
 {
 	gint ok;
 	IMAPSession *session;
 	gchar *path;
+	gchar *cache_dir;
 	gint exists, recent, unseen;
 	guint32 uid_validity;
 
@@ -1314,6 +1437,10 @@ gint imap_remove_folder(Folder *folder, FolderItem *item)
 	}
 
 	g_free(path);
+	cache_dir = folder_item_get_path(item);
+	if (is_dir_exist(cache_dir) && remove_dir_recursive(cache_dir) < 0)
+		g_warning("can't remove directory '%s'\n", cache_dir);
+	g_free(cache_dir);
 	folder_item_remove(item);
 
 	return 0;
@@ -1350,11 +1477,18 @@ static GSList *imap_get_uncached_messages(IMAPSession *session,
 			return newlist;
 		}
 		strretchomp(tmp);
-		log_print("IMAP4< %s\n", tmp);
 		if (tmp[0] != '*' || tmp[1] != ' ') {
+			log_print("IMAP4< %s\n", tmp);
 			g_free(tmp);
 			break;
 		}
+		if (strstr(tmp, "FETCH") == NULL) {
+			log_print("IMAP4< %s\n", tmp);
+			g_free(tmp);
+			continue;
+		}
+		/* log_print("IMAP4< %s\n", tmp); */
+		debug_print("IMAP4< %s\n", tmp);
 		g_string_assign(str, tmp);
 		g_free(tmp);
 
@@ -1363,6 +1497,11 @@ static GSList *imap_get_uncached_messages(IMAPSession *session,
 			log_warning(_("can't parse envelope: %s\n"), str->str);
 			continue;
 		}
+		if (item->stype == F_QUEUE) {
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_QUEUED);
+		} else if (item->stype == F_DRAFT) {
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_DRAFT);
+		}
 
 		msginfo->folder = item;
 
@@ -1449,6 +1588,8 @@ static SockInfo *imap_open(const gchar *server, gushort port, gchar *buf)
 
 #if USE_SSL
 	if (use_ssl && !ssl_init_socket(sock)) {
+		log_warning(_("Can't establish IMAP4 session with: %s:%d\n"),
+			    server, port);
 		sock_close(sock);
 		return NULL;
 	}
@@ -1457,6 +1598,8 @@ static SockInfo *imap_open(const gchar *server, gushort port, gchar *buf)
 	imap_cmd_count = 0;
 
 	if (imap_cmd_noop(sock) != IMAP_SUCCESS) {
+		log_warning(_("Can't establish IMAP4 session with: %s:%d\n"),
+			    server, port);
 		sock_close(sock);
 		return NULL;
 	}
@@ -1572,7 +1715,7 @@ static gchar *imap_get_real_path(IMAPFolder *folder, const gchar *path)
 	g_return_val_if_fail(folder != NULL, NULL);
 	g_return_val_if_fail(path != NULL, NULL);
 
-	real_path = g_strdup(path);
+	real_path = imap_locale_to_modified_utf7(path);
 	namespace = imap_find_namespace(folder, path);
 	if (namespace && namespace->separator)
 		imap_path_separator_subst(real_path, namespace->separator);
@@ -1584,10 +1727,24 @@ static gchar *imap_parse_atom(SockInfo *sock, gchar *src,
 			      gchar *dest, gint dest_len, GString *str)
 {
 	gchar *cur_pos = src;
+	gchar *nextline;
 
 	g_return_val_if_fail(str != NULL, cur_pos);
 
-	while (*cur_pos == ' ') cur_pos++;
+	/* read the next line if the current response buffer is empty */
+	while (isspace(*cur_pos)) cur_pos++;
+	while (*cur_pos == '\0') {
+		if ((nextline = sock_getline(sock)) == NULL)
+			return cur_pos;
+		g_string_assign(str, nextline);
+		cur_pos = str->str;
+		strretchomp(nextline);
+		/* log_print("IMAP4< %s\n", nextline); */
+		debug_print("IMAP4< %s\n", nextline);
+		g_free(nextline);
+
+		while (isspace(*cur_pos)) cur_pos++;
+	}
 
 	if (!strncmp(cur_pos, "NIL", 3)) {
 		*dest = '\0';
@@ -1600,21 +1757,29 @@ static gchar *imap_parse_atom(SockInfo *sock, gchar *src,
 	} else if (*cur_pos == '{') {
 		gchar buf[32];
 		gint len;
-		gchar *nextline;
+		gint line_len = 0;
 
 		cur_pos = strchr_cpy(cur_pos + 1, '}', buf, sizeof(buf));
 		len = atoi(buf);
 
-		if ((nextline = sock_getline(sock)) == NULL)
-			return cur_pos;
-		strretchomp(nextline);
-		log_print("IMAP4< %s\n", nextline);
-		g_string_assign(str, nextline);
+		g_string_truncate(str, 0);
+		cur_pos = str->str;
 
-		len = MIN(len, strlen(nextline));
-		memcpy(dest, nextline, MIN(len, dest_len - 1));
+		do {
+			if ((nextline = sock_getline(sock)) == NULL)
+				return cur_pos;
+			line_len += strlen(nextline);
+			g_string_append(str, nextline);
+			cur_pos = str->str;
+			strretchomp(nextline);
+			/* log_print("IMAP4< %s\n", nextline); */
+			debug_print("IMAP4< %s\n", nextline);
+			g_free(nextline);
+		} while (line_len < len);
+
+		memcpy(dest, cur_pos, MIN(len, dest_len - 1));
 		dest[MIN(len, dest_len - 1)] = '\0';
-		cur_pos = str->str + len;
+		cur_pos += len;
 	}
 
 	return cur_pos;
@@ -1716,8 +1881,8 @@ static MsgFlags imap_parse_flags(const gchar *flag_str)
 	while ((p = strchr(p, '\\')) != NULL) {
 		p++;
 
-		if (g_strncasecmp(p, "Recent", 6) == 0) {
-			MSG_SET_PERM_FLAGS(flags, MSG_NEW|MSG_UNREAD);
+		if (g_strncasecmp(p, "Recent", 6) == 0 && MSG_IS_UNREAD(flags)) {
+			MSG_SET_PERM_FLAGS(flags, MSG_NEW);
 		} else if (g_strncasecmp(p, "Seen", 4) == 0) {
 			MSG_UNSET_PERM_FLAGS(flags, MSG_NEW|MSG_UNREAD);
 		} else if (g_strncasecmp(p, "Deleted", 7) == 0) {
@@ -1795,8 +1960,10 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 			g_return_val_if_fail(*cur_pos == '(', NULL);
 			cur_pos = imap_parse_atom
 				(sock, cur_pos + 1, buf, sizeof(buf), line_str);
-			Xstrdup_a(date, buf, return NULL);
-			date_t = procheader_date_parse(NULL, date, 0);
+			if (buf[0] != '\0') {
+				Xstrdup_a(date, buf, return NULL);
+				date_t = procheader_date_parse(NULL, date, 0);
+			}
 
 			cur_pos = imap_parse_atom
 				(sock, cur_pos, buf, sizeof(buf), line_str);
@@ -1810,12 +1977,16 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 						     &tmp_from, &tmp_fromname,
 						     line_str);
 			g_return_val_if_fail(cur_pos != NULL, NULL);
-			Xstrdup_a(from, tmp_from,
-				  {g_free(tmp_from); g_free(tmp_fromname);
-				   return NULL;});
-			Xstrdup_a(fromname, tmp_fromname,
-				  {g_free(tmp_from); g_free(tmp_fromname);
-				   return NULL;});
+			if (tmp_from && *tmp_from != '\0')
+				Xstrdup_a(from, tmp_from,
+					  {g_free(tmp_from);
+					   g_free(tmp_fromname);
+					   return NULL;});
+			if (tmp_fromname && *tmp_fromname != '\0')
+				Xstrdup_a(fromname, tmp_fromname,
+					  {g_free(tmp_from);
+					   g_free(tmp_fromname);
+					   return NULL;});
 			g_free(tmp_from);
 			g_free(tmp_fromname);
 
@@ -1991,12 +2162,27 @@ static gint imap_select(IMAPSession *session, IMAPFolder *folder,
 {
 	gchar *real_path;
 	gint ok;
+	gint exists_, recent_, unseen_, uid_validity_;
+
+	if (!exists || !recent || !unseen || !uid_validity) {
+		if (session->mbox && strcmp(session->mbox, path) == 0)
+			return IMAP_SUCCESS;
+		exists = &exists_;
+		recent = &recent_;
+		unseen = &unseen_;
+		uid_validity = &uid_validity_;
+	}
+
+	g_free(session->mbox);
+	session->mbox = NULL;
 
 	real_path = imap_get_real_path(folder, path);
 	ok = imap_cmd_select(SESSION(session)->sock, real_path,
 			     exists, recent, unseen, uid_validity);
 	if (ok != IMAP_SUCCESS)
 		log_warning(_("can't select folder: %s\n"), real_path);
+	else
+		session->mbox = g_strdup(path);
 	g_free(real_path);
 
 	return ok;
@@ -2036,8 +2222,9 @@ catch:
 
 static gint imap_status(IMAPSession *session, IMAPFolder *folder,
 			const gchar *path,
-			gint *messages, gint *recent, gint *unseen,
-			guint32 *uid_validity)
+			gint *messages, gint *recent,
+			guint32 *uid_next, guint32 *uid_validity,
+			gint *unseen)
 {
 	gchar *real_path;
 	gchar *real_path_;
@@ -2045,14 +2232,15 @@ static gint imap_status(IMAPSession *session, IMAPFolder *folder,
 	GPtrArray *argbuf;
 	gchar *str;
 
-	*messages = *recent = *unseen = *uid_validity = 0;
+	*messages = *recent = *uid_next = *uid_validity = *unseen = 0;
 
 	argbuf = g_ptr_array_new();
 
 	real_path = imap_get_real_path(folder, path);
 	QUOTE_IF_REQUIRED(real_path_, real_path);
 	imap_cmd_gen_send(SESSION(session)->sock, "STATUS %s "
-			  "(MESSAGES RECENT UNSEEN UIDVALIDITY)", real_path_);
+			  "(MESSAGES RECENT UIDNEXT UIDVALIDITY UNSEEN)",
+			  real_path_);
 
 	ok = imap_cmd_ok(SESSION(session)->sock, argbuf);
 	if (ok != IMAP_SUCCESS) THROW(ok);
@@ -2072,12 +2260,15 @@ static gint imap_status(IMAPSession *session, IMAPFolder *folder,
 		} else if (!strncmp(str, "RECENT ", 7)) {
 			str += 7;
 			*recent = strtol(str, &str, 10);
-		} else if (!strncmp(str, "UNSEEN ", 7)) {
-			str += 7;
-			*unseen = strtol(str, &str, 10);
+		} else if (!strncmp(str, "UIDNEXT ", 8)) {
+			str += 8;
+			*uid_next = strtoul(str, &str, 10);
 		} else if (!strncmp(str, "UIDVALIDITY ", 12)) {
 			str += 12;
 			*uid_validity = strtoul(str, &str, 10);
+		} else if (!strncmp(str, "UNSEEN ", 7)) {
+			str += 7;
+			*unseen = strtol(str, &str, 10);
 		} else {
 			g_warning("invalid STATUS response: %s\n", str);
 			break;
@@ -2262,6 +2453,18 @@ static gint imap_cmd_create(SockInfo *sock, const gchar *folder)
 	return imap_cmd_ok(sock, NULL);
 }
 
+static gint imap_cmd_rename(SockInfo *sock, const gchar *old_folder,
+			    const gchar *new_folder)
+{
+	gchar *old_folder_, *new_folder_;
+
+	QUOTE_IF_REQUIRED(old_folder_, old_folder);
+	QUOTE_IF_REQUIRED(new_folder_, new_folder);
+	imap_cmd_gen_send(sock, "RENAME %s %s", old_folder_, new_folder_);
+
+	return imap_cmd_ok(sock, NULL);
+}
+
 static gint imap_cmd_delete(SockInfo *sock, const gchar *folder)
 {
 	gchar *folder_;
@@ -2284,12 +2487,15 @@ static gint imap_cmd_fetch(SockInfo *sock, guint32 uid, const gchar *filename)
 
 	imap_cmd_gen_send(sock, "UID FETCH %d BODY[]", uid);
 
-	if (sock_gets(sock, buf, sizeof(buf)) < 0)
-		return IMAP_ERROR;
-	strretchomp(buf);
-	if (buf[0] != '*' || buf[1] != ' ')
-		return IMAP_ERROR;
-	log_print("IMAP4< %s\n", buf);
+	while ((ok = imap_cmd_gen_recv(sock, buf, sizeof(buf)))
+	       == IMAP_SUCCESS) {
+		if (buf[0] != '*' || buf[1] != ' ')
+			return IMAP_ERROR;
+		if (strstr(buf, "FETCH") != NULL)
+			break;
+	}
+	if (ok != IMAP_SUCCESS)
+		return ok;
 
 	cur_pos = strchr(buf, '{');
 	g_return_val_if_fail(cur_pos != NULL, IMAP_ERROR);
@@ -2325,7 +2531,7 @@ static gint imap_cmd_append(SockInfo *sock, const gchar *destfolder,
 	g_return_val_if_fail(file != NULL, IMAP_ERROR);
 
 	size = get_file_size_as_crlf(file);
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return -1;
 	}
@@ -2562,6 +2768,215 @@ static gchar *search_array_str(GPtrArray *array, gchar *str)
 
 static void imap_path_separator_subst(gchar *str, gchar separator)
 {
-	if (separator && separator != '/')
-		subst_char(str, '/', separator);
+	gchar *p;
+	gboolean in_escape = FALSE;
+
+	if (!separator || separator == '/') return;
+
+	for (p = str; *p != '\0'; p++) {
+		if (*p == '/' && !in_escape)
+			*p = separator;
+		else if (*p == '&' && *(p + 1) != '-' && !in_escape)
+			in_escape = TRUE;
+		else if (*p == '-' && in_escape)
+			in_escape = FALSE;
+	}
+}
+
+static gchar *imap_modified_utf7_to_locale(const gchar *mutf7_str)
+{
+#if !HAVE_LIBJCONV
+	return g_strdup(mutf7_str);
+#else
+	static iconv_t cd = (iconv_t)-1;
+	static gboolean iconv_ok = TRUE;
+	GString *norm_utf7;
+	gchar *norm_utf7_p;
+	size_t norm_utf7_len;
+	const gchar *p;
+	gchar *to_str, *to_p;
+	size_t to_len;
+	gboolean in_escape = FALSE;
+
+	if (!iconv_ok) return g_strdup(mutf7_str);
+
+	if (cd == (iconv_t)-1) {
+		cd = iconv_open(conv_get_current_charset_str(), "UTF-7");
+		if (cd == (iconv_t)-1) {
+			g_warning(_("iconv cannot convert UTF-7 to %s\n"),
+				  conv_get_current_charset_str());
+			iconv_ok = FALSE;
+			return g_strdup(mutf7_str);
+		}
+	}
+
+	norm_utf7 = g_string_new(NULL);
+
+	for (p = mutf7_str; *p != '\0'; p++) {
+		/* replace: '&'  -> '+',
+			    "&-" -> '&',
+			    escaped ','  -> '/' */
+		if (!in_escape && *p == '&') {
+			if (*(p + 1) != '-') {
+				g_string_append_c(norm_utf7, '+');
+				in_escape = TRUE;
+			} else {
+				g_string_append_c(norm_utf7, '&');
+				p++;
+			}
+		} else if (in_escape && *p == ',') {
+			g_string_append_c(norm_utf7, '/');
+		} else if (in_escape && *p == '-') {
+			g_string_append_c(norm_utf7, '-');
+			in_escape = FALSE;
+		} else {
+			g_string_append_c(norm_utf7, *p);
+		}
+	}
+
+	norm_utf7_p = norm_utf7->str;
+	norm_utf7_len = norm_utf7->len;
+	to_len = strlen(mutf7_str) * 5;
+	to_p = to_str = g_malloc(to_len + 1);
+
+	if (iconv(cd, &norm_utf7_p, &norm_utf7_len, &to_p, &to_len) == -1) {
+		g_warning(_("iconv cannot convert UTF-7 to %s\n"),
+			  conv_get_current_charset_str());
+		g_string_free(norm_utf7, TRUE);
+		g_free(to_str);
+		return g_strdup(mutf7_str);
+	}
+
+	/* second iconv() call for flushing */
+	iconv(cd, NULL, NULL, &to_p, &to_len);
+	g_string_free(norm_utf7, TRUE);
+	*to_p = '\0';
+
+	return to_str;
+#endif /* !HAVE_LIBJCONV */
+}
+
+static gchar *imap_locale_to_modified_utf7(const gchar *from)
+{
+#if !HAVE_LIBJCONV
+	return g_strdup(from);
+#else
+	static iconv_t cd = (iconv_t)-1;
+	static gboolean iconv_ok = TRUE;
+	gchar *norm_utf7, *norm_utf7_p;
+	size_t from_len, norm_utf7_len;
+	GString *to_str;
+	gchar *from_tmp, *to, *p;
+	gboolean in_escape = FALSE;
+
+	if (!iconv_ok) return g_strdup(from);
+
+	if (cd == (iconv_t)-1) {
+		cd = iconv_open("UTF-7", conv_get_current_charset_str());
+		if (cd == (iconv_t)-1) {
+			g_warning(_("iconv cannot convert %s to UTF-7\n"),
+				  conv_get_current_charset_str());
+			iconv_ok = FALSE;
+			return g_strdup(from);
+		}
+	}
+
+	Xstrdup_a(from_tmp, from, return g_strdup(from));
+	from_len = strlen(from);
+	norm_utf7_len = from_len * 5;
+	Xalloca(norm_utf7, norm_utf7_len + 1, return g_strdup(from));
+	norm_utf7_p = norm_utf7;
+
+#define IS_PRINT(ch) (isprint(ch) && isascii(ch))
+
+	while (from_len > 0) {
+		if (IS_PRINT(*from_tmp)) {
+			/* printable ascii char */
+			*norm_utf7_p = *from_tmp;
+			norm_utf7_p++;
+			from_tmp++;
+			from_len--;
+		} else {
+			size_t mblen;
+
+			/* unprintable char: convert to UTF-7 */
+			for (mblen = 0;
+			     !IS_PRINT(from_tmp[mblen]) && mblen < from_len;
+			     mblen++)
+				;
+			from_len -= mblen;
+			if (iconv(cd, &from_tmp, &mblen,
+				  &norm_utf7_p, &norm_utf7_len) == -1) {
+				g_warning(_("iconv cannot convert %s to UTF-7\n"),
+					  conv_get_current_charset_str());
+				return g_strdup(from);
+			}
+
+			/* second iconv() call for flushing */
+			iconv(cd, NULL, NULL, &norm_utf7_p, &norm_utf7_len);
+		}
+	}
+
+#undef IS_PRINT
+
+	*norm_utf7_p = '\0';
+	to_str = g_string_new(NULL);
+	for (p = norm_utf7; p < norm_utf7_p; p++) {
+		/* replace: '&' -> "&-",
+			    '+' -> '&',
+			    escaped '/' -> ',' */
+		if (!in_escape && *p == '&') {
+			g_string_append(to_str, "&-");
+		} else if (!in_escape && *p == '+') {
+			g_string_append_c(to_str, '&');
+			in_escape = TRUE;
+		} else if (in_escape && *p == '/') {
+			g_string_append_c(to_str, ',');
+		} else if (in_escape && *p == '-') {
+			in_escape = FALSE;
+			g_string_append_c(to_str, '-');
+		} else {
+			g_string_append_c(to_str, *p);
+		}
+	}
+
+	if (in_escape) {
+		in_escape = FALSE;
+		g_string_append_c(to_str, '-');
+	}
+
+	to = to_str->str;
+	g_string_free(to_str, FALSE);
+
+	return to;
+#endif
+}
+
+static gboolean imap_rename_folder_func(GNode *node, gpointer data)
+{
+	FolderItem *item = node->data;
+	gchar **paths = data;
+	const gchar *oldpath = paths[0];
+	const gchar *newpath = paths[1];
+	gchar *base;
+	gchar *new_itempath;
+	gint oldpathlen;
+
+	oldpathlen = strlen(oldpath);
+	if (strncmp(oldpath, item->path, oldpathlen) != 0) {
+		g_warning("path doesn't match: %s, %s\n", oldpath, item->path);
+		return TRUE;
+	}
+
+	base = item->path + oldpathlen;
+	while (*base == G_DIR_SEPARATOR) base++;
+	if (*base == '\0')
+		new_itempath = g_strdup(newpath);
+	else
+		new_itempath = g_strconcat(newpath, G_DIR_SEPARATOR_S, base,
+					   NULL);
+	g_free(item->path);
+	item->path = new_itempath;
+
+	return FALSE;
 }
