@@ -134,8 +134,11 @@ gint imap_remove_msg			(Folder		*folder,
 					 gint		 uid);
 gint imap_remove_all_msg		(Folder		*folder,
 					 FolderItem	*item);
+gboolean imap_is_msg_changed		(Folder		*folder,
+					 FolderItem	*item,
+					 MsgInfo	*msginfo);
 
-void imap_scan_folder			(Folder		*folder,
+gint imap_scan_folder			(Folder		*folder,
 					 FolderItem	*item);
 void imap_scan_tree			(Folder		*folder);
 
@@ -144,6 +147,9 @@ gint imap_create_tree			(Folder		*folder);
 FolderItem *imap_create_folder		(Folder		*folder,
 					 FolderItem	*parent,
 					 const gchar	*name);
+gint imap_rename_folder			(Folder		*folder,
+					 FolderItem	*item,
+					 const gchar	*name);
 gint imap_remove_folder			(Folder		*folder,
 					 FolderItem	*item);
 
