@@ -134,7 +134,7 @@ static void inc_finished(MainWindow *mainwin, gboolean new_messages)
 
 	if (prefs_common.open_inbox_on_inc) {
 		item = cur_account && cur_account->inbox
-			? folder_find_item_from_path(cur_account->inbox)
+			? folder_find_item_from_identifier(cur_account->inbox)
 			: folder_get_default_inbox();
 		folderview_unselect(mainwin->folderview);
 		folderview_select(mainwin->folderview, item);
@@ -667,7 +667,7 @@ static GHashTable *inc_get_uidl_table(PrefsAccount *ac_prefs)
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			   "uidl-", ac_prefs->recv_server,
 			   "-", ac_prefs->userid, NULL);
-	if ((fp = fopen(path, "r")) == NULL) {
+	if ((fp = fopen(path, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
 		g_free(path);
 		return NULL;
@@ -697,7 +697,7 @@ static void inc_write_uidl_list(Pop3State *state)
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			   "uidl-", state->ac_prefs->recv_server,
 			   "-", state->user, NULL);
-	if ((fp = fopen(path, "w")) == NULL) {
+	if ((fp = fopen(path, "wb")) == NULL) {
 		FILE_OP_ERROR(path, "fopen");
 		g_free(path);
 		return;
@@ -833,7 +833,8 @@ gint inc_drop_message(const gchar *file, Pop3State *state)
 	gint val;
 
 	if (state->ac_prefs->inbox) {
-		inbox = folder_find_item_from_path(state->ac_prefs->inbox);
+		inbox = folder_find_item_from_identifier
+			(state->ac_prefs->inbox);
 		if (!inbox)
 			inbox = folder_get_default_inbox();
 	} else
