@@ -170,7 +170,11 @@ void messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	if (!mimeinfo) return;
 
 	file = procmsg_get_message_file_path(msginfo);
-	g_return_if_fail(file != NULL);
+	if (!file) {
+		g_warning("can't get message file path.\n");
+		procmime_mimeinfo_free(mimeinfo);
+		return;
+	}
 
 	tmpmsginfo = procheader_parse(file, msginfo->flags, TRUE);
 	headerview_show(messageview->headerview, tmpmsginfo);
@@ -179,7 +183,8 @@ void messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	textview_set_all_headers(messageview->textview, all_headers);
 	textview_set_all_headers(messageview->mimeview->textview, all_headers);
 
-	if (mimeinfo->mime_type != MIME_TEXT) {
+	if (mimeinfo->mime_type != MIME_TEXT &&
+	    mimeinfo->mime_type != MIME_TEXT_HTML) {
 		messageview_change_view_type(messageview, MVIEW_MIME);
 		mimeview_show_message(messageview->mimeview, mimeinfo, file);
 	} else {
