@@ -287,7 +287,7 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 
 	mimeview->mimeinfo = mimeinfo;
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return;
 	}
@@ -426,7 +426,7 @@ static void mimeview_show_message_part(MimeView *mimeview, MimeInfo *partinfo)
 #endif /* USE_GPGME */
 	if (!fname) return;
 
-	if ((fp = fopen(fname, "r")) == NULL) {
+	if ((fp = fopen(fname, "rb")) == NULL) {
 		FILE_OP_ERROR(fname, "fopen");
 		return;
 	}
@@ -858,7 +858,8 @@ static void mimeview_open_with(MimeView *mimeview)
 		 _("Enter the command line to open file:\n"
 		   "(`%s' will be replaced with file name)"),
 		 prefs_common.mime_open_cmd,
-		 prefs_common.mime_open_cmd_history);
+		 prefs_common.mime_open_cmd_history,
+		 TRUE);
 	if (cmd) {
 		mimeview_view_file(filename, partinfo, cmd);
 		g_free(prefs_common.mime_open_cmd);
@@ -975,7 +976,7 @@ static void mimeview_check_signature(MimeView *mimeview)
 	while (mimeinfo->parent)
 		mimeinfo = mimeinfo->parent;
 
-	if ((fp = fopen(mimeview->file, "r")) == NULL) {
+	if ((fp = fopen(mimeview->file, "rb")) == NULL) {
 		FILE_OP_ERROR(mimeview->file, "fopen");
 		return;
 	}
