@@ -190,7 +190,7 @@ GSList *procheader_get_header_list_from_file(const gchar *file)
 	FILE *fp;
 	GSList *hlist;
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return NULL;
 	}
@@ -404,7 +404,7 @@ MsgInfo *procheader_parse(const gchar *file, MsgFlags flags, gboolean full)
 
 	hentry = full ? hentry_full : hentry_short;
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return NULL;
 	}
