@@ -602,7 +602,7 @@ gint procmime_get_part(const gchar *outfile, const gchar *infile,
 	g_return_val_if_fail(infile != NULL, -1);
 	g_return_val_if_fail(mimeinfo != NULL, -1);
 
-	if ((infp = fopen(infile, "r")) == NULL) {
+	if ((infp = fopen(infile, "rb")) == NULL) {
 		FILE_OP_ERROR(infile, "fopen");
 		return -1;
 	}
@@ -611,7 +611,7 @@ gint procmime_get_part(const gchar *outfile, const gchar *infile,
 		fclose(infp);
 		return -1;
 	}
-	if ((outfp = fopen(outfile, "w")) == NULL) {
+	if ((outfp = fopen(outfile, "wb")) == NULL) {
 		FILE_OP_ERROR(outfile, "fopen");
 		fclose(infp);
 		return -1;
@@ -747,7 +747,7 @@ gboolean procmime_find_string_part(MimeInfo *mimeinfo, const gchar *filename,
 			     mimeinfo->mime_type == MIME_TEXT_HTML, FALSE);
 	g_return_val_if_fail(str != NULL, FALSE);
 
-	if ((infp = fopen(filename, "r")) == NULL) {
+	if ((infp = fopen(filename, "rb")) == NULL) {
 		FILE_OP_ERROR(filename, "fopen");
 		return FALSE;
 	}
@@ -960,7 +960,7 @@ static GList *procmime_get_mime_type_list(const gchar *file)
 	gchar *p, *delim;
 	MimeType *mime_type;
 
-	if ((fp = fopen(file, "r")) == NULL) return NULL;
+	if ((fp = fopen(file, "rb")) == NULL) return NULL;
 
 	debug_print("Reading %s ...\n", file);
 
