@@ -178,13 +178,14 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 
 	default_flags.perm_flags = MSG_NEW|MSG_UNREAD;
 	default_flags.tmp_flags = MSG_CACHED;
-	if (type == F_MH) {
+	if (type == F_MH || type == F_IMAP) {
 		if (item->stype == F_QUEUE) {
 			MSG_SET_TMP_FLAGS(default_flags, MSG_QUEUED);
 		} else if (item->stype == F_DRAFT) {
 			MSG_SET_TMP_FLAGS(default_flags, MSG_DRAFT);
 		}
-	} else if (type == F_IMAP) {
+	}
+	if (type == F_IMAP) {
 		MSG_SET_TMP_FLAGS(default_flags, MSG_IMAP);
 	} else if (type == F_NEWS) {
 		MSG_SET_TMP_FLAGS(default_flags, MSG_NEWS);
@@ -201,7 +202,7 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 		g_free(path);
 	}
 	cache_file = folder_item_get_cache_file(item);
-	if ((fp = fopen(cache_file, "r")) == NULL) {
+	if ((fp = fopen(cache_file, "rb")) == NULL) {
 		debug_print(_("\tNo cache file\n"));
 		g_free(cache_file);
 		return NULL;
@@ -385,6 +386,57 @@ void procmsg_write_flags(MsgInfo *msginfo, FILE *fp)
 	WRITE_CACHE_DATA_INT(flags, fp);
 }
 
+void procmsg_flush_mark_queue(FolderItem *item, FILE *fp)
+{
+	MsgInfo *flaginfo;
+
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(fp != NULL);
+
+	while (item->mark_queue != NULL) {
+		flaginfo = (MsgInfo *)item->mark_queue->data;
+		procmsg_write_flags(flaginfo, fp);
+		procmsg_msginfo_free(flaginfo);
+		item->mark_queue = g_slist_remove(item->mark_queue, flaginfo);
+	}
+}
+
+void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
+{
+	FILE *fp;
+	gchar *path;
+	MsgInfo msginfo;
+
+	g_return_if_fail(item != NULL);
+
+	if (item->opened) {
+		MsgInfo *queue_msginfo;
+
+		queue_msginfo = g_new0(MsgInfo, 1);
+		queue_msginfo->msgnum = num;
+		queue_msginfo->flags = flags;
+		item->mark_queue = g_slist_append
+			(item->mark_queue, queue_msginfo);
+		return;
+	}
+
+	path = folder_item_get_path(item);
+	g_return_if_fail(path != NULL);
+
+	if ((fp = procmsg_open_mark_file(path, TRUE)) == NULL) {
+		g_warning(_("can't open mark file\n"));
+		g_free(path);
+		return;
+	}
+	g_free(path);
+
+	msginfo.msgnum = num;
+	msginfo.flags = flags;
+
+	procmsg_write_flags(&msginfo, fp);
+	fclose(fp);
+}
+
 struct MarkSum {
 	gint *new;
 	gint *unread;
@@ -469,7 +521,7 @@ FILE *procmsg_open_mark_file(const gchar *folder, gboolean append)
 
 	markfile = g_strconcat(folder, G_DIR_SEPARATOR_S, MARK_FILE, NULL);
 
-	if ((fp = fopen(markfile, "r")) == NULL)
+	if ((fp = fopen(markfile, "rb")) == NULL)
 		debug_print(_("Mark file not found.\n"));
 	else if (fread(&ver, sizeof(ver), 1, fp) != 1 || MARK_VERSION != ver) {
 		debug_print(_("Mark version is different (%d != %d). "
@@ -487,12 +539,12 @@ FILE *procmsg_open_mark_file(const gchar *folder, gboolean append)
 	if (fp) {
 		/* reopen with append mode */
 		fclose(fp);
-		if ((fp = fopen(markfile, "a")) == NULL)
+		if ((fp = fopen(markfile, "ab")) == NULL)
 			g_warning(_("Can't open mark file with append mode.\n"));
 	} else {
 		/* open with overwrite mode if mark file doesn't exist or
 		   version is different */
-		if ((fp = fopen(markfile, "w")) == NULL)
+		if ((fp = fopen(markfile, "wb")) == NULL)
 			g_warning(_("Can't open mark file with write mode.\n"));
 		else {
 			ver = MARK_VERSION;
@@ -669,7 +721,7 @@ FILE *procmsg_open_message(MsgInfo *msginfo)
 		g_return_val_if_fail(file != NULL, NULL);
 	}
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		g_free(file);
 		return NULL;
@@ -760,14 +812,15 @@ void procmsg_empty_trash(void)
 	}
 }
 
-gint procmsg_send_queue(void)
+gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 {
-	FolderItem *queue;
 	gint i;
 	gint ret = 0;
 
-	queue = folder_get_default_queue();
+	if (!queue)
+		queue = folder_get_default_queue();
 	g_return_val_if_fail(queue != NULL, -1);
+
 	folder_item_scan(queue);
 	if (queue->last_num < 0) return -1;
 	else if (queue->last_num == 0) return 0;
@@ -780,8 +833,13 @@ gint procmsg_send_queue(void)
 			if (send_message_queue(file) < 0) {
 				g_warning(_("Sending queued message %d failed.\n"), i);
 				ret = -1;
-			} else
+			} else {
+				if (save_msgs)
+					procmsg_save_to_outbox
+						(queue->folder->outbox,
+						 file, TRUE);
 				folder_item_remove_msg(queue, i);
+			}
 			g_free(file);
 		}
 	}
@@ -789,6 +847,56 @@ gint procmsg_send_queue(void)
 	return ret;
 }
 
+gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file,
+			    gboolean is_queued)
+{
+	gint num;
+	FILE *fp;
+	MsgFlags flag = {0, 0};
+
+	debug_print(_("saving sent message...\n"));
+
+	if (!outbox)
+		outbox = folder_get_default_outbox();
+	g_return_val_if_fail(outbox != NULL, -1);
+
+	/* remove queueing headers */
+	if (is_queued) {
+		gchar tmp[MAXPATHLEN + 1];
+		gchar buf[BUFFSIZE];
+		FILE *outfp;
+
+		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
+			   get_rc_dir(), G_DIR_SEPARATOR, (guint)random());
+		if ((fp = fopen(file, "rb")) == NULL) {
+			FILE_OP_ERROR(file, "fopen");
+			return -1;
+		}
+		if ((outfp = fopen(tmp, "wb")) == NULL) {
+			FILE_OP_ERROR(tmp, "fopen");
+			fclose(fp);
+			return -1;
+		}
+		while (fgets(buf, sizeof(buf), fp) != NULL)
+			if (buf[0] == '\r' || buf[0] == '\n') break;
+		while (fgets(buf, sizeof(buf), fp) != NULL)
+			fputs(buf, outfp);
+		fclose(outfp);
+		fclose(fp);
+		Xstrdup_a(file, tmp, return -1);
+	}
+
+	folder_item_scan(outbox);
+	if ((num = folder_item_add_msg(outbox, file, FALSE)) < 0) {
+		g_warning(_("can't save message\n"));
+		return -1;
+	}
+
+	procmsg_add_flags(outbox, num, flag);
+
+	return 0;
+}
+
 void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline)
 {
 	static const gchar *def_cmd = "lpr %s";
@@ -808,7 +916,7 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline)
 	prtmp = g_strdup_printf("%s%cprinttmp.%08x",
 				get_mime_tmp_dir(), G_DIR_SEPARATOR, id++);
 
-	if ((prfp = fopen(prtmp, "w")) == NULL) {
+	if ((prfp = fopen(prtmp, "wb")) == NULL) {
 		FILE_OP_ERROR(prtmp, "fopen");
 		g_free(prtmp);
 		fclose(tmpfp);
