@@ -278,7 +278,7 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 	const gchar *charset = NULL;
 	GPtrArray *headers = NULL;
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return;
 	}
@@ -622,7 +622,7 @@ static gchar *make_uri_string(const gchar *bp, const gchar *ep)
 	 (ch) > 32   && \
 	 (ch) != 127 && \
 	 !isspace(ch) && \
-	 !strchr("()<>\"", (ch)))
+	 !strchr("(),;<>\"", (ch)))
 
 /* alphabet and number within 7bit ASCII */
 #define IS_ASCII_ALNUM(ch)	(isascii(ch) && isalnum(ch))
