@@ -14,11 +14,21 @@
  * 
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
-#include <alloca.h>
+#if HAVE_ALLOCA_H
+#  include <alloca.h>
+#endif
+#if HAVE_LIBJCONV
+#  include <jconv.h>
+#  include "prefs_common.h"
+#endif
 
 #include "unmime.h"
 #include "base64.h"
@@ -71,6 +81,7 @@ static int qp_char(unsigned char c1, unsigned char c2, unsigned char *c_out)
 static const char MIMEHDR_INIT[]  = "=?";	/* Start of coded sequence */
 static const char MIMEHDR_END[]   = "?=";	/* End of coded sequence */
 
+#if !HAVE_LIBJCONV
 void UnMimeHeader(unsigned char *hdr)
 {
   /* Decode a buffer containing data encoded according to RFC
@@ -235,7 +246,231 @@ void UnMimeHeader(unsigned char *hdr)
 
   *p_out = '\0';
 }
+#else /* HAVE_LIBJCONV */
+void UnMimeHeaderConv(unsigned char *hdr, unsigned char *conv_r, int conv_len)
+{
+  int  state = S_COPY_PLAIN;
+  unsigned char *p_in, *p_out, *p;
+  unsigned char enc = '\0';		/* initialization pacifies -Wall */
+  int  i;
+  unsigned char *p_mimestart = NULL;
+  iconv_t cd = (iconv_t)-1;
 
+  if (conv_r && conv_len < 0) {
+    conv_len = 0;
+    conv_r = NULL;
+  }
+  else
+    conv_len--;		/* reserve for terminating NULL character */
+
+  /* Speed up in case this is not a MIME-encoded header */
+  p = strstr(hdr, MIMEHDR_INIT);
+  if (p == NULL) {
+    /* No MIME header */
+    const char *const *codesets;
+    int n_codesets, actual_codeset, r;
+    char *newstr;
+    size_t newlen = 0;
+
+    if (prefs_common.force_charset) {
+      codesets = (const char **)&prefs_common.force_charset;
+      n_codesets = 1;
+    }
+    else
+      codesets = jconv_info_get_pref_codesets(&n_codesets);
+
+    r = jconv_alloc_conv(hdr, strlen(hdr), &newstr, &newlen,
+    			 codesets, n_codesets, &actual_codeset,
+    			 jconv_info_get_current_codeset());
+    if (r == 0) {
+      if (newlen > conv_len) newlen = conv_len;
+      strncpy(conv_r, newstr, newlen);
+      conv_r[newlen] = '\0';
+      free(newstr);
+    }
+    else {
+      strncpy(conv_r, hdr, conv_len);
+      conv_r[conv_len] = '\0';
+    }
+
+    return;
+  }
+
+  /* Loop through the buffer.
+   *  p_in : Next char to be processed.
+   *  p_out: Where to put the next processed char
+   *  enc  : Encoding used (usually, 'q' = quoted-printable)
+   */
+  for (p_out = p_in = hdr; (*p_in); ) {
+    switch (state) {
+    case S_COPY_PLAIN:
+      p = strstr(p_in, MIMEHDR_INIT);
+      if (p == NULL) {
+	/* 
+	 * No more coded data in buffer, 
+         * just move remainder into place. 
+	 */
+        i = strlen(p_in);   /* How much left */
+        if (conv_r) {
+	  int len;
+
+	  len = conv_len > i ? i : conv_len;
+	  memcpy(conv_r, p_in, len);
+	  conv_r += len;
+	  conv_len -= len;
+	}
+	memmove(p_out, p_in, i);
+	p_in += i; p_out += i;
+      }
+      else {
+	/* MIME header init found at location p */
+	if (p > p_in) {
+          /* There are some uncoded chars at the beginning. */
+          i = (p - p_in);
+	  if (conv_r) {
+	    int len;
+
+	    len = conv_len > i ? i : conv_len;
+	    memcpy(conv_r, p_in, len);
+	    conv_r += len;
+	    conv_len -= len;
+	  }
+	  memmove(p_out, p_in, i);
+	  p_out += i;
+	}
+	p_in = (p + 2);
+	state = S_SKIP_MIMEINIT;
+      }
+      break;
+
+    case S_SKIP_MIMEINIT:
+      /* Mime type definition: "charset?encoding?" */
+      p = strchr(p_in, '?');
+      if (p != NULL) {
+	/* p_in .. (p-1) holds the charset */
+	char *charset;
+
+	charset = malloc(p - p_in + 1);
+	memcpy(charset, p_in, p - p_in);
+	charset[p - p_in] = '\0';
+	if (cd != (iconv_t)-1) iconv_close(cd);
+	cd = iconv_open(jconv_info_get_current_codeset(), charset);
+	free(charset);
+
+	/* *(p+1) is the transfer encoding, *(p+2) must be a '?' */
+	if (*(p+2) == '?') {
+	  enc = tolower(*(p+1));
+	  p_in = p+3;
+	  state = S_COPY_MIME;
+	}
+	else
+	  state = S_COPY_PLAIN;
+      }
+      else
+	state = S_COPY_PLAIN;   /* Invalid data */
+      break;
+
+    case S_COPY_MIME:
+      p_mimestart = p_out;
+      p = strstr(p_in, MIMEHDR_END);  /* Find end of coded data */
+      if (p == NULL) p = p_in + strlen(p_in);
+      for (; (p_in < p); ) {
+	/* Decode all encoded data */
+	if (enc == 'q') {
+	  if (*p_in == '=') {
+	    /* Decode one char qp-coded at (p_in+1) and (p_in+2) */
+	    if (qp_char(*(p_in+1), *(p_in+2), p_out) == 0)
+	      p_in += 3;
+	    else {
+	      /* Invalid QP data - pass through unchanged. */
+	      *p_out = *p_in;
+	      p_in++;
+	    }
+	  }
+	  else if (*p_in == '_') {
+	    /* 
+             * RFC 2047: '_' inside encoded word represents 0x20.
+             * NOT a space - always the value 0x20.
+             */
+	    *p_out = 0x20;
+	    p_in++;
+	  }
+	  else {
+	    /* Copy unchanged */
+	    *p_out = *p_in;
+	    p_in++;
+	  }
+	  p_out++;
+	}
+	else if (enc == 'b') {
+	  /* Decode base64 encoded data */
+	  char delimsave;
+	  int decoded_count;
+
+	  delimsave = *p; *p = '\r';
+	  decoded_count = from64tobits(p_out, p_in);
+	  *p = delimsave;
+	  if (decoded_count > 0) 
+	    p_out += decoded_count;            
+	  p_in = p;
+	}
+	else {
+	  /* Copy unchanged */
+	  *p_out = *p_in;
+	  p_in++;
+	  p_out++;
+	}
+      }
+      if (conv_r && cd != (iconv_t)-1) {
+	size_t inleft;
+
+	inleft = p_out - p_mimestart;
+	iconv(cd, (char **)&p_mimestart, &inleft, (char **)&conv_r, &conv_len);
+      }
+      if (*p_in)
+	p_in += 2;   /* Skip the MIMEHDR_END delimiter */
+
+      /* 
+       * We've completed decoding one encoded sequence. But another
+       * may follow immediately, in which case whitespace before the
+       * new MIMEHDR_INIT delimiter must be discarded.
+       * See if that is the case 
+       */
+      p = strstr(p_in, MIMEHDR_INIT);
+      state = S_COPY_PLAIN;
+      if (p != NULL) {
+	/*
+	 * There is more MIME data later on. Is there
+         * whitespace  only before the delimiter? 
+	 */
+        unsigned char *q;
+        int  wsp_only = 1;
+
+        for (q=p_in; (wsp_only && (q < p)); q++)
+          wsp_only = isspace(*q);
+
+        if (wsp_only) {
+	  /* 
+	   * Whitespace-only before the MIME delimiter. OK,
+           * just advance p_in to past the new MIMEHDR_INIT,
+           * and prepare to process the new MIME charset/encoding
+	   * header.
+	   */
+	  p_in = p + strlen(MIMEHDR_INIT);
+	  state = S_SKIP_MIMEINIT;
+        }
+      }
+      break;
+    }
+  }
+
+  *p_out = '\0';
+  if (conv_r)
+    *conv_r = '\0';
+  if (cd != (iconv_t)-1) iconv_close(cd);
+}
+
+#endif /* !HAVE_LIBJCONV */
 
 
 /*
@@ -517,7 +752,7 @@ int MimeBodyType(unsigned char *hdrs, int WantDecode)
  * Return flag set if this line ends with a soft line-break.
  * 'bufp' is modified to point to the end of the output buffer.
  */
-static int DoOneQPLine(unsigned char **bufp, flag delimited, flag issoftline)
+int DoOneQPLine(unsigned char **bufp, flag delimited, flag issoftline)
 {
   unsigned char *buf = *bufp;
   unsigned char *p_in, *p_out, *p;
@@ -528,10 +763,10 @@ static int DoOneQPLine(unsigned char **bufp, flag delimited, flag issoftline)
    * Special case: line consists of a single =2E and messages are 
    * dot-terminated.  Line has to be dot-stuffed after decoding.
    */
-  if (delimited && !issoftline && buf[0]=='=' && !strncmp(*bufp, "=2E\r\n", 5))
+  if (delimited && !issoftline && buf[0]=='=' && !strncmp(*bufp, "=2E\n", 4))
   {
-      strcpy(buf, "..\r\n");
-      *bufp += 5;
+      strcpy(buf, "..\n");
+      *bufp += 4;
       return(FALSE);
   }
 
@@ -599,6 +834,7 @@ static int DoOneQPLine(unsigned char **bufp, flag delimited, flag issoftline)
  * 'bufp' is modified to point to the end of the output buffer.
  */
 
+#if 0
 int UnMimeBodyline(unsigned char **bufp, flag delimited, flag softline)
 {
   unsigned char *buf = *bufp;
@@ -651,6 +887,7 @@ int UnMimeBodyline(unsigned char **bufp, flag delimited, flag softline)
 
   return ret;
 }
+#endif /* 0 */
 
 
 #ifdef STANDALONE
@@ -681,9 +918,9 @@ int main(int argc, char *argv[])
 
   pid = getpid();
   sprintf(fnam, "/tmp/i_unmime.%x", pid);
-  fd_orig = fopen(fnam, "w");
+  fd_orig = fopen(fnam, "wb");
   sprintf(fnam, "/tmp/o_unmime.%x", pid);
-  fd_conv = fopen(fnam, "w");
+  fd_conv = fopen(fnam, "wb");
 #endif
 
   BufSize = BUFSIZE_INCREMENT;    /* Initial size of buffer */
