@@ -1111,6 +1111,30 @@ gchar **strsplit_with_quote(const gchar *str, const gchar *delim,
 	return str_array;
 }
 
+gchar *get_abbrev_newsgroup_name(const gchar *group)
+{
+	gchar *abbrev_group;
+	gchar *ap;
+	const gchar *p = group;
+
+	abbrev_group = ap = g_malloc(strlen(group) + 1);
+
+	while (*p) {
+		while (*p == '.')
+			*ap++ = *p++;
+		if (strchr(p, '.')) {
+			*ap++ = *p++;
+			while (*p != '.') p++;
+		} else {
+			strcpy(ap, p);
+			return abbrev_group;
+		}
+	}
+
+	*ap = '\0';
+	return abbrev_group;
+}
+
 GList *uri_list_extract_filenames(const gchar *uri_list)
 {
 	GList *result = NULL;
@@ -1334,7 +1358,7 @@ off_t get_file_size_as_crlf(const gchar *file)
 	off_t size = 0;
 	gchar buf[BUFFSIZE];
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return -1;
 	}
@@ -1411,6 +1435,18 @@ gboolean is_dir_exist(const gchar *dir)
 	return FALSE;
 }
 
+gboolean is_file_entry_exist(const gchar *file)
+{
+	struct stat s;
+
+	if (stat(file, &s) < 0) {
+		if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 gint change_dir(const gchar *dir)
 {
 	gchar *prevdir = NULL;
@@ -1651,7 +1687,7 @@ gint copy_file(const gchar *src, const gchar *dest)
 	gchar *dest_bak = NULL;
 	gboolean err = FALSE;
 
-	if ((src_fp = fopen(src, "r")) == NULL) {
+	if ((src_fp = fopen(src, "rb")) == NULL) {
 		FILE_OP_ERROR(src, "fopen");
 		return -1;
 	}
@@ -1665,7 +1701,7 @@ gint copy_file(const gchar *src, const gchar *dest)
 		}
 	}
 
-	if ((dest_fp = fopen(dest, "w")) == NULL) {
+	if ((dest_fp = fopen(dest, "wb")) == NULL) {
 		FILE_OP_ERROR(dest, "fopen");
 		fclose(src_fp);
 		if (dest_bak) {
@@ -2061,7 +2097,7 @@ static FILE *log_fp = NULL;
 void set_log_file(const gchar *filename)
 {
 	if (log_fp) return;
-	log_fp = fopen(filename, "w");
+	log_fp = fopen(filename, "wb");
 	if (!log_fp)
 		FILE_OP_ERROR(filename, "fopen");
 }
