@@ -270,6 +270,8 @@ gchar **strsplit_with_quote		(const gchar	*str,
 					 const gchar	*delim,
 					 gint		 max_tokens);
 
+gchar *get_abbrev_newsgroup_name	(const gchar	*group);
+
 GList *uri_list_extract_filenames	(const gchar	*uri_list);
 void decode_uri				(gchar		*decoded_uri,
 					 const gchar	*encoded_uri);
@@ -288,9 +290,15 @@ gchar *get_domain_name		(void);
 off_t get_file_size		(const gchar	*file);
 off_t get_file_size_as_crlf	(const gchar	*file);
 off_t get_left_file_size	(FILE		*fp);
+
 gboolean file_exist		(const gchar	*file,
 				 gboolean	 allow_fifo);
 gboolean is_dir_exist		(const gchar	*dir);
+gboolean is_file_entry_exist	(const gchar	*file);
+
+#define is_file_exist(file)		file_exist(file, FALSE)
+#define is_file_or_fifo_exist(file)	file_exist(file, TRUE)
+
 gint change_dir			(const gchar	*dir);
 gint make_dir_hier		(const gchar	*dir);
 gint remove_all_files		(const gchar	*dir);
@@ -308,9 +316,6 @@ gint change_file_mode_rw	(FILE		*fp,
 FILE *my_tmpfile		(void);
 FILE *str_open_as_stream	(const gchar	*str);
 
-#define is_file_exist(file)		file_exist(file, FALSE)
-#define is_file_or_fifo_exist(file)	file_exist(file, TRUE)
-
 /* process execution */
 gint execute_async		(gchar *const	 argv[]);
 gint execute_sync		(gchar *const	 argv[]);
